package cn.com.qmth.examcloud.core.examwork.api.controller;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.PathUtil;
import cn.com.qmth.examcloud.commons.web.helpers.page.PageInfo;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.basic.api.bean.OrgBean;
import cn.com.qmth.examcloud.core.basic.api.bean.StudentBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetCourseReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetStudentReq;
import cn.com.qmth.examcloud.core.basic.api.request.InsertOrUpdateStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetCourseResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetOrgResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetStudentResp;
import cn.com.qmth.examcloud.core.basic.api.response.InsertOrUpdateStudentResp;
import cn.com.qmth.examcloud.core.examwork.api.controller.bean.ExamStudentDomain;
import cn.com.qmth.examcloud.core.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.examwork.service.ExamStudentService;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamStudentInfo;
import io.swagger.annotations.ApiOperation;

/**
 * 考生服务API Created by songyue on 17/1/13.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.ctr.examwork}/exam_student")
public class ExamStudentController extends ControllerSupport {

	@Autowired
	ExamStudentRepo examStudentRepo;

	@Autowired
	ExamStudentService examStudentService;

	@Autowired
	ExamRepo examRepo;

	@Autowired
	StudentCloudService studentCloudService;

	@Autowired
	CourseCloudService courseCloudService;

	@Autowired
	OrgCloudService orgCloudService;

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param studentId
	 * @return
	 */
	@ApiOperation(value = "查询学生的所有考试")
	@GetMapping("/byStudentId/{studentId}")
	public List<ExamStudentEntity> getExamStudentListByStudentId(@PathVariable Long studentId) {

		return examStudentRepo.findByStudentId(studentId);
	}

	/**
	 * 重构
	 *
	 * @author WANGWEI
	 * @param examCriteria
	 * @param curPage
	 * @param pageSize
	 * @return
	 */
	@ApiOperation(value = "查询考试学生带条件和分页", notes = "带条件带分页")
	@GetMapping("/all/{curPage}/{pageSize}")
	public PageInfo<ExamStudentDomain> getExamStudentList(
			@ModelAttribute ExamStudentDomain examCriteria, @PathVariable Integer curPage,
			@PathVariable Integer pageSize) {
		User accessUser = getAccessUser();
		Specification<ExamStudentEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.equal(root.get("rootOrgId"), accessUser.getRootOrgId()));

			if (null != examCriteria.getOrgId()) {
				predicates.add(cb.equal(root.get("orgId"), examCriteria.getOrgId()));
			}

			if (null != examCriteria.getExamId()) {
				predicates.add(cb.equal(root.get("examId"), examCriteria.getExamId()));
			}
			if (StringUtils.isNotEmpty(examCriteria.getStudentName())) {
				predicates.add(cb.like(root.get("name"),
						toSqlSearchPattern(examCriteria.getStudentName())));
			}
			if (StringUtils.isNotEmpty(examCriteria.getStudentCode())) {
				predicates.add(cb.like(root.get("studentCode"),
						toSqlSearchPattern(examCriteria.getStudentCode())));
			}
			if (StringUtils.isNotEmpty(examCriteria.getCourseCode())) {
				predicates.add(cb.equal(root.get("courseCode"), examCriteria.getCourseCode()));
			}
			if (StringUtils.isNotEmpty(examCriteria.getCourseLevel())) {
				predicates.add(cb.equal(root.get("courseLevel"), examCriteria.getCourseLevel()));
			}
			if (StringUtils.isNotEmpty(examCriteria.getCourseName())) {
				predicates.add(cb.like(root.get("courseName"),
						toSqlSearchPattern(examCriteria.getCourseName())));
			}
			if (!StringUtils.isEmpty(examCriteria.getExamSite())) {
				predicates.add(cb.like(root.get("examSite"),
						toSqlSearchPattern(examCriteria.getExamSite())));
			}
			if (StringUtils.isNotEmpty(examCriteria.getIdentityNumber())) {
				predicates.add(cb.like(root.get("identityNumber"),
						toSqlSearchPattern(examCriteria.getIdentityNumber())));
			}
			if (StringUtils.isNotEmpty(examCriteria.getSpecialtyName())) {
				predicates.add(cb.like(root.get("specialtyName"),
						toSqlSearchPattern(examCriteria.getSpecialtyName())));
			}
			if (StringUtils.isNotEmpty(examCriteria.getInfoCollector())) {
				predicates.add(cb.like(root.get("infoCollector"),
						toSqlSearchPattern(examCriteria.getInfoCollector())));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		PageRequest pageRequest = new PageRequest(curPage, pageSize,
				new Sort(Direction.DESC, "updateTime"));

		Page<ExamStudentEntity> examStudents = examStudentRepo.findAll(specification, pageRequest);

		List<ExamStudentDomain> ret = Lists.newArrayList();

		for (ExamStudentEntity cur : examStudents) {
			ExamEntity exam = examRepo.findOne(cur.getExamId());
			GetCourseReq req = new GetCourseReq();
			req.setId(cur.getCourseId());
			req.setRootOrgId(accessUser.getRootOrgId());
			GetCourseResp getCourseResp = courseCloudService.getCourse(req);
			CourseBean courseBean = getCourseResp.getCourseBean();
			GetOrgReq getOrgReq = new GetOrgReq();
			getOrgReq.setOrgId(cur.getOrgId());
			GetOrgResp getOrgResp = orgCloudService.getOrg(getOrgReq);
			OrgBean org = getOrgResp.getOrg();

			GetStudentReq getStudentReq = new GetStudentReq();
			getStudentReq.setRootOrgId(accessUser.getRootOrgId());
			getStudentReq.setIdentityNumber(cur.getIdentityNumber());
			GetStudentResp getStudentResp = studentCloudService.getStudent(getStudentReq);
			StudentBean studentBean = getStudentResp.getStudentInfo();

			ExamStudentDomain bean = new ExamStudentDomain();
			bean.setId(cur.getId());
			bean.setExamId(exam.getId());
			bean.setExamName(exam.getName());
			bean.setStudentId(cur.getStudentId());
			bean.setStudentName(cur.getName());
			bean.setStudentCode(cur.getStudentCode());
			bean.setIdentityNumber(cur.getIdentityNumber());
			bean.setCourseCode(courseBean.getCode());
			bean.setCourseName(courseBean.getName());
			bean.setInfoCollector(cur.getInfoCollector());
			bean.setOrgId(cur.getOrgId());
			bean.setOrgCode(org.getCode());
			bean.setOrgName(org.getName());
			bean.setPaperType(cur.getPaperType());
			bean.setPhone(studentBean.getPhoneNumber());
			bean.setGrade(cur.getGrade());
			bean.setSpecialtyName(cur.getSpecialtyName());
			bean.setExamSite(cur.getExamSite());
			bean.setExamType(exam.getExamType().name());
			bean.setUpdateTime(cur.getUpdateTime());
			ret.add(bean);
		}
		return new PageInfo<ExamStudentDomain>(examStudents, ret);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param id
	 * @return
	 */
	@ApiOperation(value = "按ID查询考试学生", notes = "ID查询")
	@GetMapping("{id}")
	public ExamStudentEntity getExamStudentById(@PathVariable Long id) {
		ExamStudentEntity es = examStudentRepo.findOne(id);
		if (null == es) {
			throw new StatusException("E-520001", "考生不存在");
		}
		return es;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examStudent
	 * @return
	 */
	@Transactional
	@ApiOperation(value = "新增考试学生", notes = "新增")
	@PostMapping()
	public ExamStudentDomain addExamStudent(@RequestBody ExamStudentDomain examStudent) {
		trim(examStudent);
		Long rootOrgId = getRootOrgId();
		examStudent.setRootOrgId(rootOrgId);
		examStudent.setInfoCollector(examStudent.getInfoCollector());
		return saveExamStudent(examStudent);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examStudent
	 * @return
	 */
	@Transactional
	@ApiOperation(value = "更新考试学生", notes = "更新")
	@PutMapping()
	public ExamStudentDomain updateExamStudent(@RequestBody ExamStudentDomain examStudent) {
		trim(examStudent);
		Long rootOrgId = getRootOrgId();
		examStudent.setRootOrgId(rootOrgId);
		examStudent.setInfoCollector(examStudent.getInfoCollector());
		return saveExamStudent(examStudent);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examStudent
	 * @return
	 */
	private ExamStudentDomain saveExamStudent(ExamStudentDomain examStudent) {
		InsertOrUpdateStudentReq saveStudentReq = new InsertOrUpdateStudentReq();
		saveStudentReq.setIdentityNumber(examStudent.getIdentityNumber());
		saveStudentReq.setName(examStudent.getStudentName());
		saveStudentReq.setPhoneNumber(examStudent.getPhone());
		saveStudentReq.setOrgCode(examStudent.getOrgCode());
		saveStudentReq.setOrgId(examStudent.getOrgId());
		saveStudentReq.setOrgName(examStudent.getOrgName());
		saveStudentReq.setRemark(examStudent.getRemark());
		saveStudentReq.setRootOrgId(examStudent.getRootOrgId());
		saveStudentReq.setStudentCode(examStudent.getStudentCode());

		InsertOrUpdateStudentResp insertOrUpdateStudentResp = studentCloudService
				.insertOrUpdateStudent(saveStudentReq);

		Long studentId = insertOrUpdateStudentResp.getStudentId();

		ExamStudentInfo info = new ExamStudentInfo();
		info.setCourseCode(examStudent.getCourseCode());
		info.setCourseLevel(examStudent.getCourseLevel());
		info.setCourseName(examStudent.getCourseName());
		info.setExamId(examStudent.getExamId());
		info.setIdentityNumber(examStudent.getIdentityNumber());
		info.setPaperType(examStudent.getPaperType());
		info.setRootOrgId(examStudent.getRootOrgId());
		info.setStudentCode(examStudent.getStudentCode());
		info.setStudentName(examStudent.getStudentName());
		info.setStudentId(studentId);
		info.setInfoCollector(examStudent.getInfoCollector());
		info.setGrade(examStudent.getGrade());
		info.setExamSite(examStudent.getExamSite());
		info.setSpecialtyName(examStudent.getSpecialtyName());
		info.setRemark(examStudent.getRemark());

		ExamStudentInfo savedExamStudent = examStudentService.saveExamStudent(info);

		ExamStudentDomain ret = new ExamStudentDomain();
		ret.setId(savedExamStudent.getId());
		ret.setStudentId(studentId);
		ret.setStudentCode(savedExamStudent.getStudentCode());
		ret.setStudentName(savedExamStudent.getStudentName());
		ret.setCourseCode(savedExamStudent.getCourseCode());
		ret.setCourseName(savedExamStudent.getCourseName());
		return ret;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param ids
	 * @return
	 */
	@Transactional
	@ApiOperation(value = "按ID删除考试学生", notes = "删除")
	@DeleteMapping("/{ids}")
	public List<Long> deleteExamStudent(@PathVariable String ids) {
		List<Long> idList = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		examStudentService.deleteExamStudentsByStudentIds(idList);
		return idList;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examId
	 * @return
	 */
	@ApiOperation(value = "按考试删除考试学生", notes = "按考试删除")
	@Transactional
	@DeleteMapping("/exam/{examId}")
	public Long deleteExamStudents(@PathVariable Long examId) {
		examStudentService.deleteExamStudentsByExamId(examId);
		return examId;
	}

	@ApiOperation(value = "下载导入模板", notes = "下载导入模板")
	@GetMapping("/downloadTemplate")
	public void downloadTemplate(HttpServletResponse response) {
		String resoucePath = PathUtil.getResoucePath("studentImportTemplate.xlsx");
		exportFile("考生导入模板.xlsx", new File(resoucePath));
	}

	@ApiOperation(value = "复制考试学生", notes = "复制")
	@Transactional
	@PostMapping("/copy/{sourceExamId}/{targetExamId}")
	public void copyExamStudents(@PathVariable Long sourceExamId, @PathVariable Long targetExamId) {
		// 任务模块处理
	}

}
