package cn.com.qmth.examcloud.core.examwork.base.enums;

import java.util.HashMap;
import java.util.Map;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年8月6日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public enum ExamProperty {

	/**
	 * 是否允许上传附件(离线考试)
	 */
	CAN_UPLOAD_ATTACHMENT(31L, "是否允许上传附件(离线考试)"),
	/**
	 * 是否显示客观题成绩
	 */
	IS_OBJ_SCORE_VIEW(30L, "是否显示客观题成绩"),
	/**
	 * IP白名单
	 */
	IP_ADDRESSES(29L, "IP白名单"),
	/**
	 * 是否IP限制
	 */
	IP_LIMIT(28L, "是否IP限制"),
	/**
	 * 活体检测结束分钟数
	 */
	FACE_VERIFY_END_MINUTE(27L, "活体检测结束分钟数"),
	/**
	 * 活体检测开始分钟数
	 */
	FACE_VERIFY_START_MINUTE(26L, "活体检测开始分钟数"),
	/**
	 * 是否开启人脸活体检测
	 */
	IS_FACE_VERIFY(25L, "是否开启人脸活体检测"),
	/**
	 * 阅卷方式
	 */
	MARKING_TYPE(24L, "阅卷方式"),
	/**
	 * 预警阈值
	 */
	WARN_THRESHOLD(23L, "预警阈值"),
	/**
	 * 抓拍间隔
	 */
	SNAPSHOT_INTERVAL(21L, "抓拍间隔"),
	/**
	 * 进入考试是否验证人脸识别(强制、非强制)
	 */
	IS_FACE_CHECK(21L, "进入考试是否验证人脸识别(强制、非强制)"),
	/**
	 * 是否启用人脸识别
	 */
	IS_FACE_ENABLE(20L, "是否启用人脸识别"),
	/**
	 * 套题补充说明
	 */
	nestedAnswerRemark(19L, "套题补充说明"),
	/**
	 * 问答题补充说明
	 */
	TEXT_ANSWER_REMARK(18L, "问答题补充说明"),
	/**
	 * 填空题补充说明
	 */
	FILL_BLANK_REMARK(17L, "填空题补充说明"),
	/**
	 * 判断题补充说明
	 */
	BOOL_ANSWER_REMARK(16L, "判断题补充说明"),
	/**
	 * 多选题补充说明
	 */
	MUTIPLE_ANSWER_REMARK(15L, "多选题补充说明"),
	/**
	 * 单选题补充说明
	 */
	SINGLE_ANSWER_REMARK(14L, "单选题补充说明"),
	/**
	 * 填空题补充说明是否可填
	 */
	FILL_BLANK_EDIT(13L, "填空题补充说明是否可填"),
	/**
	 * 判断题补充说明是否可填
	 */
	BOOL_EDIT(12L, "判断题补充说明是否可填"),
	/**
	 * 多选题补充说明是否可填
	 */
	MUTIPLE_EDIT(11L, "多选题补充说明是否可填"),
	/**
	 * 单选题补充说明是否可填
	 */
	SINGLE_EDIT(10L, "单选题补充说明是否可填"),
	/**
	 * 练习模式
	 */
	PRACTICE_TYPE(9L, "练习模式"),
	/**
	 * 作弊说明
	 */
	CHEATING_REMARK(8L, "作弊说明"),
	/**
	 * 是否展示作弊
	 */
	SHOW_CHEATING_REMARK(7L, "是否展示作弊"),
	/**
	 * 考后说明
	 */
	AFTER_EXAM_REMARK(6L, "考后说明"),
	/**
	 * 考前说明
	 */
	BEFORE_EXAM_REMARK(5L, "考前说明"),
	/**
	 * 断点续考时间(秒)
	 */
	EXAM_RECONNECT_TIME(4L, "断点续考时间(秒)"),
	/**
	 * 交卷冻结时间
	 */
	FREEZE_TIME(3L, "交卷冻结时间"),
	/**
	 * 是否入学考试
	 */
	IS_ENTRANCE_EXAM(2L, "是否入学考试"),

	/**
	 * 发布成绩
	 */
	SCORE_PUBLISHING(1L, "发布成绩");

	// ===========================================================================

	/**
	 * key ID
	 */
	private Long keyId;

	/**
	 * 描述
	 */
	private String desc;

	private static Map<Long, ExamProperty> map = new HashMap<Long, ExamProperty>();

	static {
		for (ExamProperty e : ExamProperty.values()) {
			map.put(e.keyId, e);
		}
	}

	/**
	 * 构造函数
	 *
	 * @param keyId
	 * @param desc
	 */
	private ExamProperty(Long keyId, String desc) {
		this.keyId = keyId;
		this.desc = desc;
	}

	/**
	 * get ExamProperty by keyId
	 *
	 * @author WANGWEI
	 * @param keyId
	 * @return
	 */
	public static ExamProperty getByKeyId(Long keyId) {
		return map.get(keyId);
	}

	public Long getKeyId() {
		return keyId;
	}

	public String getDesc() {
		return desc;
	}

}
