package cn.com.qmth.examcloud.core.examwork.service.impl;

import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.basic.api.bean.OrgBean;
import cn.com.qmth.examcloud.core.basic.api.bean.StudentBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetStudentReq;
import cn.com.qmth.examcloud.core.basic.api.request.SaveCourseReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetOrgResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetStudentResp;
import cn.com.qmth.examcloud.core.basic.api.response.SaveCourseResp;
import cn.com.qmth.examcloud.core.examwork.dao.ExamOrgRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.examwork.service.ExamStudentService;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamStudentInfo;

/**
 * 考试学生服务类 Created by songyue on 17/1/14.
 */
@Service
public class ExamStudentServiceImpl implements ExamStudentService {

	protected static final Logger log = LoggerFactory.getLogger(ExamStudentServiceImpl.class);

	@Autowired
	ExamStudentRepo examStudentRepo;

	@Autowired
	ExamRepo examRepo;

	@Autowired
	ExamOrgRepo examOrgRepo;

	@Autowired
	OrgCloudService orgCloudService;

	@Autowired
	ExamStudentServiceImpl examStudentService;

	@Autowired
	StudentCloudService studentCloudService;

	@Autowired
	CourseCloudService courseCloudService;

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param ids
	 */
	@Override
	public void deleteExamStudentsByStudentIds(List<Long> ids) {
		List<ExamStudentEntity> examStudents = examStudentRepo.findByIdIn(ids);
		for (ExamStudentEntity examStudent : examStudents) {
			// 网考判断
			if (System.currentTimeMillis() > 0) {
				throw new StatusException("E-150112", examStudent.getName() + "已开始考试,不能删除");
			}
			examStudentRepo.delete(examStudent);
		}
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examId
	 */
	@Override
	public void deleteExamStudentsByExamId(Long examId) {
		ExamEntity exam = examRepo.findOne(examId);
		// 已经开考
		if (exam.getBeginTime().before(new Date())) {
			throw new StatusException("E-150113", "该考试已开始,不能删除");
		}
		examStudentRepo.deleteByExamId(examId);
	}

	/*
	 * 实现
	 *
	 * @author WANGWEI
	 * 
	 * @see cn.com.qmth.examcloud.core.examwork.service.ExamStudentService#
	 * saveExamStudent(cn.com.qmth.examcloud.core.examwork.service.bean.
	 * ExamStudentInfo)
	 */
	@Override
	public ExamStudentInfo saveExamStudent(ExamStudentInfo examStudentInfo) {
		String paperType = examStudentInfo.getPaperType();
		if (StringUtils.isBlank(paperType)) {
			paperType = "X";
		} else if (!paperType.matches("[A-Z]")) {
			throw new StatusException("E-100020", "试卷类型必须为一个大写的英文字母");
		}

		Long rootOrgId = examStudentInfo.getRootOrgId();
		GetOrgReq getOrgReq = new GetOrgReq();
		getOrgReq.setOrgId(rootOrgId);
		GetOrgResp getOrgResp = orgCloudService.getOrg(getOrgReq);
		OrgBean rootOrg = getOrgResp.getOrg();
		if (null != rootOrg.getParentId()) {
			throw new StatusException("E-100001", "rootOrgId is wrong");
		}

		ExamEntity exam = null;
		Long examId = examStudentInfo.getExamId();
		if (null == examId) {
			String examName = examStudentInfo.getExamName();
			if (StringUtils.isBlank(examName)) {
				throw new StatusException("E-100002", "examId & examName are both blank");
			}
			exam = examRepo.findByNameAndRootOrgId(examName, rootOrgId);
			if (null == exam) {
				throw new StatusException("E-100005", "考试不存在");
			}
		} else {
			exam = examRepo.findOne(examId);
		}

		String studentCode = examStudentInfo.getStudentCode();
		String identityNumber = examStudentInfo.getIdentityNumber();

		if (StringUtils.isBlank(identityNumber)) {
			throw new StatusException("E-100003", "identityNumber is null");
		}
		GetStudentReq getStudentReq = new GetStudentReq();
		getStudentReq.setRootOrgId(rootOrgId);
		getStudentReq.setStudentCode(studentCode);
		getStudentReq.setIdentityNumber(identityNumber);
		GetStudentResp getStudentResp = studentCloudService.getStudent(getStudentReq);

		StudentBean studentInfo = getStudentResp.getStudentInfo();

		String courseCode = examStudentInfo.getCourseCode();
		String courseName = examStudentInfo.getCourseName();
		String courseLevel = examStudentInfo.getCourseLevel();

		SaveCourseReq saveCourseReq = new SaveCourseReq();
		saveCourseReq.setCourseId(examStudentInfo.getCourseId());
		saveCourseReq.setCourseCode(courseCode);
		saveCourseReq.setCourseName(courseName);
		saveCourseReq.setCourseLevel(courseLevel);
		saveCourseReq.setRootOrgId(rootOrgId);

		SaveCourseResp saveCourseResp = courseCloudService.saveCourse(saveCourseReq);
		CourseBean courseBean = saveCourseResp.getCourseBean();

		ExamStudentEntity examStudent = examStudentRepo.findByExamIdAndStudentIdAndCourseId(
				exam.getId(), studentInfo.getId(), courseBean.getId());

		if (null == examStudent) {
			examStudent = new ExamStudentEntity();
		}

		examStudent.setInfoCollector(examStudentInfo.getInfoCollector());
		examStudent.setName(studentInfo.getName());
		examStudent.setRootOrgId(rootOrgId);

		examStudent.setCourseId(courseBean.getId());

		examStudent.setExamId(exam.getId());
		examStudent.setIdentityNumber(studentInfo.getIdentityNumber());
		examStudent.setStudentCode(studentInfo.getStudentCode());
		examStudent.setPaperType(paperType);
		examStudent.setStudentId(studentInfo.getId());
		examStudent.setOrgId(studentInfo.getOrgId());
		examStudent.setGrade(examStudentInfo.getGrade());
		examStudent.setSpecialtyName(examStudentInfo.getSpecialtyName());
		examStudent.setExamSite(examStudentInfo.getExamSite());
		examStudent.setRemark(examStudentInfo.getRemark());

		ExamStudentEntity saved = examStudentRepo.saveAndFlush(examStudent);

		ExamStudentInfo ret = new ExamStudentInfo();
		ret.setId(saved.getId());
		ret.setCourseCode(courseBean.getCode());
		ret.setCourseLevel(courseBean.getLevel());
		ret.setCourseName(courseBean.getName());
		ret.setExamId(exam.getId());
		ret.setExamName(exam.getName());
		ret.setIdentityNumber(saved.getIdentityNumber());
		ret.setStudentCode(saved.getStudentCode());
		ret.setPaperType(saved.getPaperType());
		ret.setRootOrgId(saved.getRootOrgId());
		ret.setStudentName(studentInfo.getName());

		return ret;
	}
}
